"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class TripCreatedV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VEHICLE_ID_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    VIN_FIELD_NUMBER: builtins.int
    vehicle_id: builtins.str
    user_id: builtins.str
    vin: builtins.str
    def __init__(
        self,
        *,
        vehicle_id: builtins.str | None = ...,
        user_id: builtins.str | None = ...,
        vin: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["user_id", b"user_id", "vehicle_id", b"vehicle_id", "vin", b"vin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["user_id", b"user_id", "vehicle_id", b"vehicle_id", "vin", b"vin"]) -> None: ...

global___TripCreatedV1 = TripCreatedV1

class TripCreatedV2(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VEHICLE_ID_FIELD_NUMBER: builtins.int
    VIN_FIELD_NUMBER: builtins.int
    vehicle_id: builtins.str
    vin: builtins.str
    def __init__(
        self,
        *,
        vehicle_id: builtins.str | None = ...,
        vin: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["vehicle_id", b"vehicle_id", "vin", b"vin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["vehicle_id", b"vehicle_id", "vin", b"vin"]) -> None: ...

global___TripCreatedV2 = TripCreatedV2

class DeviceCreatedV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEVICE_ID_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    device_id: builtins.str
    user_id: builtins.str
    def __init__(
        self,
        *,
        device_id: builtins.str | None = ...,
        user_id: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["device_id", b"device_id", "user_id", b"user_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["device_id", b"device_id", "user_id", b"user_id"]) -> None: ...

global___DeviceCreatedV1 = DeviceCreatedV1

class VehicleCreatedV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VEHICLE_ID_FIELD_NUMBER: builtins.int
    vehicle_id: builtins.str
    def __init__(
        self,
        *,
        vehicle_id: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["vehicle_id", b"vehicle_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["vehicle_id", b"vehicle_id"]) -> None: ...

global___VehicleCreatedV1 = VehicleCreatedV1



import threading
from concurrent.futures import Future
from typing import Union, Dict, Optional, Generator, List

from hedwig.backends.base import HedwigPublisherBaseBackend, HedwigConsumerBaseBackend
from hedwig.models import Message


class MockHedwigPublisherBackend(HedwigPublisherBaseBackend):
    def _mock_queue_message(self, message: Message):
        pass

    def _publish(self, message: Message, payload: Union[str, bytes], attributes: Dict[str, str]) -> Union[str, Future]:
        pass


class MockHedwigConsumerBackend(HedwigConsumerBaseBackend):
    def message_attributes(self, queue_message) -> dict:
        pass

    def extend_visibility_timeout(self, visibility_timeout_s: int, metadata) -> None:
        pass

    def requeue_dead_letter(self, num_messages: int = 10, visibility_timeout: Optional[int] = None) -> None:
        pass

    def pull_messages(
        self,
        num_messages: int = 10,
        visibility_timeout: Optional[int] = None,
        shutdown_event: Optional[threading.Event] = None,
    ) -> Union[Generator, List]:
        pass

    def process_message(self, queue_message) -> None:
        pass

    def ack_message(self, queue_message) -> None:
        pass

    def nack_message(self, queue_message) -> None:
        pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: schema.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'schema.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedwig.protobuf import options_pb2 as hedwig_dot_protobuf_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cschema.proto\x12\x0fhedwig_examples\x1a\x1dhedwig/protobuf/options.proto\"8\n\rUserCreatedV1\x12\x0f\n\x07user_id\x18\x01 \x01(\t:\x16\x9a\x82\x19\x12\x08\x01\x10\x00\x1a\x0cuser-created\"8\n\rUserUpdatedV1\x12\x0f\n\x07user_id\x18\x01 \x01(\t:\x16\x9a\x82\x19\x12\x08\x01\x10\x00\x1a\x0cuser-updatedb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'schema_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_USERCREATEDV1']._loaded_options = None
  _globals['_USERCREATEDV1']._serialized_options = b'\232\202\031\022\010\001\020\000\032\014user-created'
  _globals['_USERUPDATEDV1']._loaded_options = None
  _globals['_USERUPDATEDV1']._serialized_options = b'\232\202\031\022\010\001\020\000\032\014user-updated'
  _globals['_USERCREATEDV1']._serialized_start=64
  _globals['_USERCREATEDV1']._serialized_end=120
  _globals['_USERUPDATEDV1']._serialized_start=122
  _globals['_USERUPDATEDV1']._serialized_end=178
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: hedwig/protobuf/container.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'hedwig/protobuf/container.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fhedwig/protobuf/container.proto\x12\x06hedwig\x1a\x19google/protobuf/any.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb0\x01\n\nMetadataV1\x12\x11\n\tpublisher\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x07headers\x18\x03 \x03(\x0b\x32\x1f.hedwig.MetadataV1.HeadersEntry\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x89\x01\n\tPayloadV1\x12\x16\n\x0e\x66ormat_version\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12$\n\x08metadata\x18\x03 \x01(\x0b\x32\x12.hedwig.MetadataV1\x12\x0e\n\x06schema\x18\x04 \x01(\t\x12\"\n\x04\x64\x61ta\x18\x05 \x01(\x0b\x32\x14.google.protobuf.AnyBI\n\x1cio.github.cloudchacho.hedwigZ)github.com/cloudchacho/hedwig-go/protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'hedwig.protobuf.container_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034io.github.cloudchacho.hedwigZ)github.com/cloudchacho/hedwig-go/protobuf'
  _globals['_METADATAV1_HEADERSENTRY']._loaded_options = None
  _globals['_METADATAV1_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_METADATAV1']._serialized_start=104
  _globals['_METADATAV1']._serialized_end=280
  _globals['_METADATAV1_HEADERSENTRY']._serialized_start=234
  _globals['_METADATAV1_HEADERSENTRY']._serialized_end=280
  _globals['_PAYLOADV1']._serialized_start=283
  _globals['_PAYLOADV1']._serialized_end=420
# @@protoc_insertion_point(module_scope)



import os

from examples.base_settings import *  # noqa

HEDWIG_CONSUMER_BACKEND = 'hedwig.backends.aws.AWSSQSConsumerBackend'
HEDWIG_PUBLISHER_BACKEND = 'hedwig.backends.aws.AWSSNSPublisherBackend'

if 'AWS_ACCESS_KEY' in os.environ:
    AWS_ACCESS_KEY = os.environ['AWS_ACCESS_KEY']
AWS_ACCOUNT_ID = os.environ['AWS_ACCOUNT_ID']
AWS_REGION = os.environ['AWS_REGION']
if 'AWS_SECRET_KEY' in os.environ:
    AWS_SECRET_KEY = os.environ['AWS_SECRET_KEY']

HEDWIG_QUEUE = 'DEV-MYAPP'



import os

from examples.base_settings import *  # noqa

HEDWIG_CONSUMER_BACKEND = 'hedwig.backends.redis.RedisStreamsConsumerBackend'
HEDWIG_PUBLISHER_BACKEND = 'hedwig.backends.redis.RedisStreamsPublisherBackend'

REDIS_URL = os.environ['REDIS_URL']

HEDWIG_SUBSCRIPTIONS = ['dev-user-created-v1']

HEDWIG_QUEUE = 'dev:myapp'

HEDWIG_VISIBILITY_TIMEOUT_S = 20

HEDWIG_MAX_DELIVERY_ATTEMPTS = 1



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: protobuf_minor_versioned.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'protobuf_minor_versioned.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedwig.protobuf import options_pb2 as hedwig_dot_protobuf_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eprotobuf_minor_versioned.proto\x12\x15tests.minor_versioned\x1a\x1dhedwig/protobuf/options.proto\"K\n\rTripCreatedV1\x12\x12\n\nvehicle_id\x18\x01 \x02(\t\x12\x0f\n\x07user_id\x18\x02 \x02(\t\x12\x0b\n\x03vin\x18\x03 \x02(\t:\x08\x9a\x82\x19\x04\x08\x01\x10\x01\"8\n\rTripCreatedV2\x12\x12\n\nvehicle_id\x18\x01 \x02(\t\x12\x0b\n\x03vin\x18\x02 \x02(\t:\x06\x9a\x82\x19\x02\x08\x02\"=\n\x0f\x44\x65viceCreatedV1\x12\x11\n\tdevice_id\x18\x01 \x02(\t\x12\x0f\n\x07user_id\x18\x02 \x02(\t:\x06\x9a\x82\x19\x02\x08\x01\".\n\x10VehicleCreatedV1\x12\x12\n\nvehicle_id\x18\x01 \x02(\t:\x06\x9a\x82\x19\x02\x08\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protobuf_minor_versioned_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TRIPCREATEDV1']._loaded_options = None
  _globals['_TRIPCREATEDV1']._serialized_options = b'\232\202\031\004\010\001\020\001'
  _globals['_TRIPCREATEDV2']._loaded_options = None
  _globals['_TRIPCREATEDV2']._serialized_options = b'\232\202\031\002\010\002'
  _globals['_DEVICECREATEDV1']._loaded_options = None
  _globals['_DEVICECREATEDV1']._serialized_options = b'\232\202\031\002\010\001'
  _globals['_VEHICLECREATEDV1']._loaded_options = None
  _globals['_VEHICLECREATEDV1']._serialized_options = b'\232\202\031\002\010\001'
  _globals['_TRIPCREATEDV1']._serialized_start=88
  _globals['_TRIPCREATEDV1']._serialized_end=163
  _globals['_TRIPCREATEDV2']._serialized_start=165
  _globals['_TRIPCREATEDV2']._serialized_end=221
  _globals['_DEVICECREATEDV1']._serialized_start=223
  _globals['_DEVICECREATEDV1']._serialized_end=284
  _globals['_VEHICLECREATEDV1']._serialized_start=286
  _globals['_VEHICLECREATEDV1']._serialized_end=332
# @@protoc_insertion_point(module_scope)



import base64
import dataclasses
import threading
import uuid
from concurrent.futures import Future
from typing import Union, Dict, Optional, Generator, List, Tuple

from redis import Redis

from hedwig.backends.base import HedwigPublisherBaseBackend, HedwigConsumerBaseBackend
from hedwig.conf import settings
from hedwig.models import Message


def _client():
    return Redis.from_url(settings.REDIS_URL, protocol=3)


@dataclasses.dataclass(frozen=True)
class RedisMessage:
    stream: bytes
    key: bytes
    payload: dict[bytes, bytes]
    delivery_attempt: int


@dataclasses.dataclass(frozen=True)
class RedisMetadata:
    id: str
    """
    Redis message id
    """

    stream: str
    """
    Redis stream name where message was consumed from
    """

    delivery_attempt: int
    """
    The delivery attempt counter.
    The first delivery of a given message will have this value as 1. The value
    is calculated as best effort and is approximate.
    """


class RedisStreamsPublisherBackend(HedwigPublisherBaseBackend):
    def __init__(self) -> None:
        self._r = _client()

    def _mock_queue_message(self, message: Message) -> RedisMessage:
        payload, attributes = message.serialize()
        if isinstance(payload, bytes):
            payload = base64.encodebytes(payload).decode()
            attributes['hedwig_encoding'] = 'base64'
        redis_message = {"hedwig_payload": payload, **attributes}
        redis_message = {k.encode(): v.encode() for k, v in redis_message.items()}
        stream = f"hedwig:{self.topic(message)}".encode()
        message_id = f"{attributes['hedwig_message_timestamp']}-0".encode()
        return RedisMessage(stream=stream, key=message_id, payload=redis_message, delivery_attempt=1)

    def _publish(self, message: Message, payload: Union[str, bytes], attributes: Dict[str, str]) -> Union[str, Future]:
        key = f"hedwig:{self.topic(message)}"
        # Redis requires UTF-8 encoded strings
        if isinstance(payload, bytes):
            payload = base64.encodebytes(payload).decode()
            attributes['hedwig_encoding'] = 'base64'
        redis_message = {"hedwig_payload": payload, **attributes}
        message_id = self._r.xadd(key, redis_message)
        return message_id


class RedisStreamsConsumerBackend(HedwigConsumerBaseBackend):
    def __init__(self, dlq=False) -> None:
        assert (
            settings.HEDWIG_MAX_DELIVERY_ATTEMPTS
        ), "HEDWIG_MAX_DELIVERY_ATTEMPTS must be set for RedisStreamsConsumerBackend"
        assert (
            settings.HEDWIG_VISIBILITY_TIMEOUT_S
        ), "HEDWIG_VISIBILITY_TIMEOUT_S must be set for RedisStreamsConsumerBackend"
        self._r = _client()
        self._group = settings.HEDWIG_QUEUE
        self._consumer_id = str(uuid.uuid4())
        self._main_stream = f"hedwig:{settings.HEDWIG_QUEUE}"
        self._deadletter_stream = f"hedwig:{settings.HEDWIG_QUEUE}:dlq"
        if dlq:
            self._streams = [self._deadletter_stream]
        else:
            self._streams = [f"hedwig:{x}" for x in settings.HEDWIG_SUBSCRIPTIONS]
            # main queue for DLQ re-queued messages
            self._streams.append(self._main_stream)
        super().__init__()

    def message_attributes(self, queue_message: RedisMessage) -> dict:
        return {k.decode(): v.decode() for k, v in queue_message.payload.items()}

    def extend_visibility_timeout(self, visibility_timeout_s: int, metadata) -> None:
        assert visibility_timeout_s == settings.HEDWIG_VISIBILITY_TIMEOUT_S, "Visibility timeout is not configurable"
        # reset idle time to 0, thereby extending visibility timeout
        self._r.xclaim(
            name=metadata.stream,
            groupname=self._group,
            consumername=self._consumer_id,
            min_idle_time=0,
            message_ids=[metadata.id],
            justid=True,
        )

    def requeue_dead_letter(self, num_messages: int = 10, visibility_timeout: Optional[int] = None) -> None:
        total_count = 0
        while True:
            dlq_entries: Tuple[str, List[Tuple[str, Dict]], List] = self._r.xautoclaim(
                name=self._deadletter_stream,
                groupname=self._group,
                consumername=self._consumer_id,
                min_idle_time=0,
                start_id="0-0",
                count=num_messages,
            )
            if not dlq_entries:
                break
            count = 0
            messages = dlq_entries[1]
            if not messages:
                break

            count += len(messages)
            total_count += count
            self._requeue_messages(messages)
            print(f"Requeued {count} messages")
        while True:
            entries: Dict[str, List[List[Tuple[str, Dict]]]] = self._r.xreadgroup(
                self._group,
                self._consumer_id,
                streams={self._deadletter_stream: ">"},
                count=num_messages,
                block=500,
            )
            if not entries:
                break

            messages = []
            for stream, stream_entries in entries.items():
                for msgs in stream_entries:
                    if not msgs:
                        continue
                    messages.extend(msgs)

            count = len(messages)
            total_count += count
            if count == 0:
                break
            self._requeue_messages(messages)
            print(f"Requeued {count} messages")

        print('-' * 80)
        print(f"Requeued total {total_count} messages")

    def _requeue_messages(self, messages):
        with self._r.pipeline() as pipeline:
            message_ids = []
            for message in messages:
                message_ids.append(message[0])
                pipeline.xadd(self._main_stream, message[1])
            pipeline.xack(self._deadletter_stream, self._group, *message_ids)
            pipeline.execute()

    def _process_raw_messages(
        self, stream: bytes, messages: list[tuple[bytes, dict[bytes, bytes]]]
    ) -> Generator[RedisMessage, None, None]:
        for message in messages:
            message_id, message_payload = message
            # manual tracking of delivery attempts
            metadata = self._r.xpending_range(stream, self._group, message_id, message_id, 1)[0]
            if metadata["times_delivered"] > settings.HEDWIG_MAX_DELIVERY_ATTEMPTS:
                with self._r.pipeline() as pipeline:
                    pipeline.xadd(self._deadletter_stream, message_payload)
                    pipeline.xack(stream, self._group, message_id)
                    pipeline.execute()
                continue
            yield RedisMessage(
                stream=stream,
                key=message_id,
                payload=message_payload,
                delivery_attempt=metadata["times_delivered"],
            )

    def pull_messages(  # type: ignore[return]
        self,
        num_messages: int = 10,
        visibility_timeout: Optional[int] = None,
        shutdown_event: Optional[threading.Event] = None,
    ) -> Union[Generator, List]:
        assert not visibility_timeout, "Visibility timeout is not configurable"
        try:
            # claim timed out messages
            for stream in self._streams:
                stream_entries = self._r.xautoclaim(
                    name=stream,
                    groupname=self._group,
                    consumername=self._consumer_id,
                    min_idle_time=int(settings.HEDWIG_VISIBILITY_TIMEOUT_S * 1000),
                    start_id="0-0",
                    count=num_messages,
                )[1]
                yield from self._process_raw_messages(stream.encode(), stream_entries)
            entries: dict[bytes, list[list[tuple[bytes, dict[bytes, bytes]]]]] = self._r.xreadgroup(
                groupname=self._group,
                consumername=self._consumer_id,
                streams={x: ">" for x in self._streams},
                count=num_messages,
                block=500,
            )
            if not entries:
                return []
            for stream_, stream_entries in entries.items():
                for messages in stream_entries:
                    yield from self._process_raw_messages(stream_, messages)
        finally:
            self._perform_error_counter_inactivity_reset()
            self._call_heartbeat_hook()

    def process_message(self, queue_message: RedisMessage) -> None:
        stream = queue_message.stream.decode()
        message_id = queue_message.key.decode()
        fields = self.message_attributes(queue_message)
        # body is always UTF-8 string
        message_payload = fields.pop("hedwig_payload")
        if fields.pop("hedwig_encoding", None) == "base64":
            message_payload = base64.decodebytes(message_payload.encode())
        self.message_handler(
            message_payload,
            fields,
            RedisMetadata(message_id, stream, queue_message.delivery_attempt),
        )

    def ack_message(self, queue_message: RedisMessage) -> None:
        self._r.xack(queue_message.stream, self._group, queue_message.key)

    def nack_message(self, queue_message: RedisMessage) -> None:
        # let visibility timeout take care of it
        pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: protobuf_bad.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'protobuf_bad.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedwig.protobuf import options_pb2 as hedwig_dot_protobuf_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12protobuf_bad.proto\x12\ttests.bad\x1a\x1dhedwig/protobuf/options.proto\"\'\n\rDeviceCreated\x12\x0e\n\x06\x66oobar\x18\x01 \x02(\x05:\x06\x9a\x82\x19\x02\x08\x02\"8\n\rTripCreatedV3\x12\x11\n\tdevice_id\x18\x01 \x02(\t:\x14\x9a\x82\x19\x10\x08\x02\x1a\x0ctrip_created\"3\n\x0bTripCreated\x12\x0e\n\x06\x66oobar\x18\x01 \x02(\x05:\x14\x9a\x82\x19\x10\x08\x01\x1a\x0ctrip_created\"\x1f\n\rTripCreatedV4\x12\x0e\n\x06\x66oobar\x18\x01 \x02(\x05')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protobuf_bad_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DEVICECREATED']._loaded_options = None
  _globals['_DEVICECREATED']._serialized_options = b'\232\202\031\002\010\002'
  _globals['_TRIPCREATEDV3']._loaded_options = None
  _globals['_TRIPCREATEDV3']._serialized_options = b'\232\202\031\020\010\002\032\014trip_created'
  _globals['_TRIPCREATED']._loaded_options = None
  _globals['_TRIPCREATED']._serialized_options = b'\232\202\031\020\010\001\032\014trip_created'
  _globals['_DEVICECREATED']._serialized_start=64
  _globals['_DEVICECREATED']._serialized_end=103
  _globals['_TRIPCREATEDV3']._serialized_start=105
  _globals['_TRIPCREATEDV3']._serialized_end=161
  _globals['_TRIPCREATED']._serialized_start=163
  _globals['_TRIPCREATED']._serialized_end=214
  _globals['_TRIPCREATEDV4']._serialized_start=216
  _globals['_TRIPCREATEDV4']._serialized_end=247
# @@protoc_insertion_point(module_scope)



import os

from examples.base_settings import *  # noqa

GOOGLE_APPLICATION_CREDENTIALS = os.environ.get('GOOGLE_APPLICATION_CREDENTIALS')
GOOGLE_CLOUD_PROJECT = os.environ.get('GOOGLE_CLOUD_PROJECT')

HEDWIG_CONSUMER_BACKEND = 'hedwig.backends.gcp.GooglePubSubConsumerBackend'
HEDWIG_PUBLISHER_BACKEND = 'hedwig.backends.gcp.GooglePubSubPublisherBackend'

HEDWIG_SUBSCRIPTIONS = ['dev-user-created-v1']

HEDWIG_QUEUE = 'dev-myapp'



import logging

import examples.init_gcp
from hedwig import consumer


def main():
    examples.init_gcp.init()

    logging.info("Starting Hedwig consumer")
    consumer.listen_for_messages(num_messages=1)


if __name__ == "__main__":
    main()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.descriptor_pb2
import google.protobuf.internal.extension_dict
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MessageOptions(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAJOR_VERSION_FIELD_NUMBER: builtins.int
    MINOR_VERSION_FIELD_NUMBER: builtins.int
    MESSAGE_TYPE_FIELD_NUMBER: builtins.int
    major_version: builtins.int
    """major version for this message type. this is required"""
    minor_version: builtins.int
    """minor version for this message type. this is optional since 0 is a valid minor version"""
    message_type: builtins.str
    """message type explicit declaration. this is optional and will be automatically
    inferred from message name assuming this format: `<MessageType>V<MajorVersion>`.
    """
    def __init__(
        self,
        *,
        major_version: builtins.int | None = ...,
        minor_version: builtins.int | None = ...,
        message_type: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["major_version", b"major_version", "message_type", b"message_type", "minor_version", b"minor_version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["major_version", b"major_version", "message_type", b"message_type", "minor_version", b"minor_version"]) -> None: ...

global___MessageOptions = MessageOptions

MESSAGE_OPTIONS_FIELD_NUMBER: builtins.int
message_options: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, global___MessageOptions]



from contextlib import contextmanager
from typing import Dict, Iterator

from opentelemetry import trace
from opentelemetry.trace import Span

from hedwig.instrumentation.compat import Getter, extract, inject
from hedwig.models import Message

getter = Getter()


@contextmanager
def on_receive(attributes: dict) -> Iterator[Span]:
    """
    Hook for instrumenting consumer after message is dequeued. If applicable, starts a new span.
    :param attributes: Message attributes received from the backend, this is used to extract trace context.
    :return:
    """
    tracectx = extract(getter, attributes)  # type: ignore

    tracer = trace.get_tracer(__name__)
    with tracer.start_as_current_span("message_received", context=tracectx, kind=trace.SpanKind.CONSUMER) as span:
        yield span


def on_message(message: Message) -> None:
    """
    Hook for instrumenting consumer after message is deserialized and validated. If applicable, updates the current span
    with the right name.
    :param message:
    :return:
    """
    span = trace.get_current_span()
    span.update_name(message.type)


@contextmanager
def on_publish(message: Message, headers: Dict) -> Iterator[Span]:
    """
    Hook for instrumenting publish. If applicable, injects tracing headers into headers dictionary.
    :param message:
    :param headers:
    :return:
    """
    tracer = trace.get_tracer(__name__)
    with tracer.start_as_current_span(f"publish/{message.type}", kind=trace.SpanKind.PRODUCER) as span:
        inject(dict.__setitem__, headers)
        yield span



import base64
import threading
from time import sleep
from unittest import mock

import freezegun
import pytest

from hedwig.commands import requeue_dead_letter
from hedwig.models import Message

try:
    from hedwig.backends.redis import RedisMetadata, RedisMessage
except ImportError:
    pass
from hedwig.exceptions import ValidationError, CallbackNotFound
from tests.models import MessageType

redis = pytest.importorskip('hedwig.backends.redis')


@pytest.fixture(autouse=True)
def redis_settings(settings):
    settings.REDIS_URL = "redis://redis:6379/10"
    settings.HEDWIG_PUBLISHER_BACKEND = "hedwig.backends.redis.RedisStreamsPublisherBackend"
    settings.HEDWIG_CONSUMER_BACKEND = "hedwig.backends.redis.RedisStreamsConsumerBackend"
    settings.HEDWIG_VISIBILITY_TIMEOUT_S = 10
    settings.HEDWIG_MAX_DELIVERY_ATTEMPTS = 1
    settings.HEDWIG_QUEUE = "dev:myapp"
    settings.HEDWIG_SUBSCRIPTIONS = ['dev-trip-created-v1']
    yield settings


@pytest.fixture
def redis_client():
    return redis._client()


@pytest.fixture(autouse=True)
def setup_redis_stream_groups(redis_client, redis_settings):
    for hedwig_subscription in redis_settings.HEDWIG_SUBSCRIPTIONS:
        redis_client.xgroup_create(
            name=f"hedwig:{hedwig_subscription}",
            groupname=redis_settings.HEDWIG_QUEUE,
            mkstream=True,
        )
    redis_client.xgroup_create(
        name=f"hedwig:{redis_settings.HEDWIG_QUEUE}",
        groupname=redis_settings.HEDWIG_QUEUE,
        mkstream=True,
    )
    redis_client.xgroup_create(
        name=f"hedwig:{redis_settings.HEDWIG_QUEUE}:dlq",
        groupname=redis_settings.HEDWIG_QUEUE,
        mkstream=True,
    )
    yield
    for hedwig_subscription in redis_settings.HEDWIG_SUBSCRIPTIONS:
        redis_client.xgroup_destroy(
            name=f"hedwig:{hedwig_subscription}",
            groupname=redis_settings.HEDWIG_QUEUE,
        )
        redis_client.delete(f"hedwig:{hedwig_subscription}")
    redis_client.xgroup_destroy(
        name=f"hedwig:{redis_settings.HEDWIG_QUEUE}",
        groupname=redis_settings.HEDWIG_QUEUE,
    )
    redis_client.delete(f"hedwig:{redis_settings.HEDWIG_QUEUE}")
    redis_client.xgroup_destroy(
        name=f"hedwig:{redis_settings.HEDWIG_QUEUE}:dlq",
        groupname=redis_settings.HEDWIG_QUEUE,
    )
    redis_client.delete(f"hedwig:{redis_settings.HEDWIG_QUEUE}:dlq")


def assert_redis_message_payload(message: Message, redis_message_payload: dict[bytes, bytes]):
    expected_hedwig_payload, expected_attributes = message.serialize()
    # convert bytes to strings
    attributes = {k.decode(): v.decode() for k, v in redis_message_payload.items()}
    hedwig_payload = attributes.pop("hedwig_payload")
    if attributes.pop("hedwig_encoding", None) == "base64":
        hedwig_payload = base64.decodebytes(hedwig_payload.encode())
    assert attributes == expected_attributes
    assert hedwig_payload == expected_hedwig_payload


@freezegun.freeze_time("2025-02-18")
class TestPubSubPublisher:
    def test_publish_success(self, message, redis_client):
        redis_publisher = redis.RedisStreamsPublisherBackend()
        message_id = redis_publisher.publish(message)

        stream = f"hedwig:{redis_publisher.topic(message)}".encode()
        resp = redis_client.xread(streams={stream: "0-0"})
        assert len(resp) == 1
        assert stream in resp
        messages = resp[stream]
        assert len(messages) == 1
        msg_id, msg_payload = messages[0][0]
        assert msg_id == message_id
        assert_redis_message_payload(message, msg_payload)

    @mock.patch('tests.handlers._trip_created_handler', autospec=True)
    def test_sync_mode(self, callback_mock, message_factory, redis_settings):
        redis_settings.HEDWIG_SYNC = True
        publisher = redis.RedisStreamsPublisherBackend()
        message = message_factory(msg_type=MessageType.trip_created)
        stream = f"hedwig:{publisher.topic(message)}"
        message_id = "1739836800000-0"

        message.publish()
        callback_mock.assert_called_once_with(
            message.with_provider_metadata(RedisMetadata(id=message_id, stream=stream, delivery_attempt=1))
        )

    def test_sync_mode_detects_invalid_callback(self, redis_settings, message_factory):
        redis_settings.HEDWIG_SYNC = True

        message = message_factory(msg_type=MessageType.vehicle_created)
        with pytest.raises(ValidationError) as exc_info:
            message.publish()
        assert isinstance(exc_info.value.__context__, CallbackNotFound)


pre_process_hook = mock.MagicMock()
post_process_hook = mock.MagicMock()
heartbeat_hook = mock.MagicMock()


@pytest.fixture(name='prepost_process_hooks')
def _prepost_process_hooks(redis_settings):
    redis_settings.HEDWIG_HEARTBEAT_HOOK = 'tests.test_backends.test_redis.heartbeat_hook'
    redis_settings.HEDWIG_PRE_PROCESS_HOOK = 'tests.test_backends.test_redis.pre_process_hook'
    redis_settings.HEDWIG_POST_PROCESS_HOOK = 'tests.test_backends.test_redis.post_process_hook'
    yield
    pre_process_hook.reset_mock(side_effect=True)
    post_process_hook.reset_mock(side_effect=True)
    heartbeat_hook.reset_mock(side_effect=True)


def _shutdown(fn):
    """
    decorator for pull_messages() method to set shutdown event
    and break from the infinite loop in fetch_and_process_messages
    """

    def wrapper(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        finally:
            kwargs["shutdown_event"].set()

    return wrapper


def _publish_and_consume_message(message: Message):
    msg_id = message.publish()
    shutdown_event = threading.Event()
    redis_consumer = redis.RedisStreamsConsumerBackend()
    redis_consumer.pull_messages = _shutdown(redis_consumer.pull_messages)
    redis_consumer.fetch_and_process_messages(num_messages=1, shutdown_event=shutdown_event)
    return msg_id


class TestRedisConsumer:
    def test_pull_messages(self, message, prepost_process_hooks):
        message_id = message.publish()

        redis_consumer = redis.RedisStreamsConsumerBackend()
        items = list(redis_consumer.pull_messages(num_messages=1))

        assert len(items) == 1
        redis_message: RedisMessage = items[0]
        assert redis_message.stream == b'hedwig:dev-trip-created-v1'
        assert redis_message.key == message_id
        assert_redis_message_payload(message, redis_message.payload)
        assert redis_message.delivery_attempt == 1
        heartbeat_hook.assert_called_once_with(error_count=0)

        # assert there is no available messages since above message is currently in PEL (Pending Entries List)
        items = list(redis_consumer.pull_messages(num_messages=1))
        assert len(items) == 0

    def test_pull_messages_with_expired_visibility_timeout(self, message, prepost_process_hooks, redis_settings):
        redis_settings.HEDWIG_VISIBILITY_TIMEOUT_S = 0.3
        redis_settings.HEDWIG_MAX_DELIVERY_ATTEMPTS = 3
        message_id = message.publish()

        redis_consumer = redis.RedisStreamsConsumerBackend()
        items = list(redis_consumer.pull_messages(num_messages=1))

        assert len(items) == 1
        redis_message: RedisMessage = items[0]
        assert redis_message.stream == b'hedwig:dev-trip-created-v1'
        assert redis_message.key == message_id
        assert_redis_message_payload(message, redis_message.payload)
        assert redis_message.delivery_attempt == 1
        heartbeat_hook.assert_called_once_with(error_count=0)

        sleep(0.5)
        heartbeat_hook.reset_mock()
        # create new consumer and auto-claim message, which visibility timeout was expired
        redis_consumer = redis.RedisStreamsConsumerBackend()
        items = list(redis_consumer.pull_messages(num_messages=1))

        assert len(items) == 1
        redis_message = items[0]
        assert redis_message.stream == b'hedwig:dev-trip-created-v1'
        assert redis_message.key == message_id
        assert_redis_message_payload(message, redis_message.payload)
        assert redis_message.delivery_attempt == 2
        heartbeat_hook.assert_called_once_with(error_count=0)

    def test_pull_messages_and_move_to_dlq(self, message, prepost_process_hooks, redis_settings, redis_client):
        redis_settings.HEDWIG_VISIBILITY_TIMEOUT_S = 0.3
        # 2nd delivery attempt will be moved to dlq
        redis_settings.HEDWIG_MAX_DELIVERY_ATTEMPTS = 1
        message_id = message.publish()

        redis_consumer = redis.RedisStreamsConsumerBackend()
        items = list(redis_consumer.pull_messages(num_messages=1))

        assert len(items) == 1
        redis_message: RedisMessage = items[0]
        assert redis_message.stream == b'hedwig:dev-trip-created-v1'
        assert redis_message.key == message_id
        assert_redis_message_payload(message, redis_message.payload)
        assert redis_message.delivery_attempt == 1
        heartbeat_hook.assert_called_once_with(error_count=0)

        sleep(0.5)
        heartbeat_hook.reset_mock()
        # because max delivery attempt is set to 1, 2nd attempt will move message to dlq
        items = list(redis_consumer.pull_messages(num_messages=1))
        assert len(items) == 0

        # validate message exists in dlq and matches original message
        dlq_stream = redis_consumer._deadletter_stream.encode()
        entries = redis_client.xreadgroup(
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            streams={dlq_stream: ">"},
            count=1,
        )
        assert len(entries) == 1
        assert dlq_stream in entries
        _, msg_payload = entries[dlq_stream][0][0]
        assert_redis_message_payload(message, msg_payload)

        # validate original message is no longer available in origin stream,
        # because it was confirmed with XACK.
        entries = redis_client.xautoclaim(
            name="hedwig:dev-trip-created-v1",
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            min_idle_time=0,
            start_id="0-0",
            count=1,
            justid=True,
        )
        assert len(entries) == 0

    def test_success_requeue_dead_letter(self, message, redis_client, redis_settings):
        redis_settings.HEDWIG_VISIBILITY_TIMEOUT_S = 0.1
        message.publish()

        # move valid message to dlq
        redis_consumer = redis.RedisStreamsConsumerBackend()
        items = list(redis_consumer.pull_messages(num_messages=1))
        assert len(items) == 1
        sleep(0.5)
        redis_consumer = redis.RedisStreamsConsumerBackend()
        items = list(redis_consumer.pull_messages(num_messages=1))
        assert len(items) == 0
        # claimed messaged - requeue dlq scenario 1
        dlq_stream = redis_consumer._deadletter_stream.encode()
        dlq_entries = redis_client.xreadgroup(
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            streams={dlq_stream: ">"},
            count=1,
        )
        assert len(dlq_entries) == 1

        # available dlq message - requeue dlq scenario 2
        payload, attributes = message.serialize()
        if isinstance(payload, bytes):
            payload = base64.encodebytes(payload).decode()
            attributes['hedwig_encoding'] = 'base64'
        redis_message = {"hedwig_payload": payload, **attributes}
        redis_client.xadd(dlq_stream, redis_message)

        requeue_dead_letter()

        _, dlq_entries, _ = redis_client.xautoclaim(
            name=dlq_stream,
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            min_idle_time=0,
            start_id="0-0",
            count=1,
        )
        assert len(dlq_entries) == 0

        main_stream = redis_consumer._main_stream.encode()
        entries = redis_client.xreadgroup(
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            streams={main_stream: ">"},
            count=10,
        )
        assert len(entries[main_stream][0]) == 2
        for _, msg_payload in entries[main_stream][0]:
            assert_redis_message_payload(message, msg_payload)

    def test_fetch_and_process_messages_success(
        self, message_factory, redis_client, redis_settings, prepost_process_hooks
    ):
        def message_handler(message: Message):
            assert message.id == trip_created_message.id
            assert message.data == trip_created_message.data

        redis_settings.HEDWIG_CALLBACKS = {('trip_created', '1.*'): message_handler}
        trip_created_message = message_factory(msg_type=MessageType.trip_created)
        _publish_and_consume_message(trip_created_message)

        pre_process_hook.assert_called_once_with()
        post_process_hook.assert_called_once_with()
        heartbeat_hook.assert_called_once_with(error_count=0)
        # assert that XACK was successfully sent and there is no message available
        _, entries, _ = redis_client.xautoclaim(
            name=b'hedwig:dev-trip-created-v1',
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            min_idle_time=0,
            start_id="0-0",
            count=1,
        )
        assert len(entries) == 0

    def test_fetch_and_process_messages_failure(
        self, message_factory, redis_client, redis_settings, prepost_process_hooks
    ):
        def message_handler(message: Message):
            raise Exception("error during message processing")

        redis_settings.HEDWIG_CALLBACKS = {('trip_created', '1.*'): message_handler}
        trip_created_message = message_factory(msg_type=MessageType.trip_created)
        msg_id = _publish_and_consume_message(trip_created_message)

        pre_process_hook.assert_called_once_with()
        post_process_hook.assert_not_called()
        heartbeat_hook.assert_called_once_with(error_count=1)
        # assert that XACK was not sent and message is still available
        _, entries, _ = redis_client.xautoclaim(
            name=b'hedwig:dev-trip-created-v1',
            groupname=redis_settings.HEDWIG_QUEUE,
            consumername="test-client",
            min_idle_time=0,
            start_id="0-0",
            count=1,
        )
        assert len(entries) == 1
        assert entries[0][0] == msg_id

    def test_success_extend_visibility_timeout(
        self, message_factory, redis_client, redis_settings, prepost_process_hooks
    ):
        def message_handler(message: Message):
            sleep(1)
            message.extend_visibility_timeout(redis_settings.HEDWIG_VISIBILITY_TIMEOUT_S)
            # raise error to avoid sending XACK to keep message in PEL (Pending Entries List)
            raise Exception("error during message processing")

        redis_settings.HEDWIG_CALLBACKS = {('trip_created', '1.*'): message_handler}
        trip_created_message = message_factory(msg_type=MessageType.trip_created, id=str(message_factory))
        msg_id = _publish_and_consume_message(trip_created_message)

        heartbeat_hook.assert_called_once_with(error_count=1)
        metadata = redis_client.xpending_range(
            "hedwig:dev-trip-created-v1", redis_settings.HEDWIG_QUEUE, msg_id, msg_id, 1
        )
        assert metadata[0]["time_since_delivered"] < 500

    def test_extend_visibility_timeout_invalid_timeout_value(self, message, redis_settings):
        with pytest.raises(AssertionError) as err:
            message.extend_visibility_timeout(1000)

        assert "Visibility timeout is not configurable" in str(err.value)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MetadataV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class HeadersEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    PUBLISHER_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    HEADERS_FIELD_NUMBER: builtins.int
    publisher: builtins.str
    """Message publisher service"""
    @property
    def timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Publish timestamp in epoch milliseconds (integer)"""
    @property
    def headers(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Custom headers associated with the message"""
    def __init__(
        self,
        *,
        publisher: builtins.str = ...,
        timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        headers: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["timestamp", b"timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["headers", b"headers", "publisher", b"publisher", "timestamp", b"timestamp"]) -> None: ...

global___MetadataV1 = MetadataV1

class PayloadV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FORMAT_VERSION_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    SCHEMA_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    format_version: builtins.str
    """Format version for the message"""
    id: builtins.str
    """Message identifier"""
    @property
    def metadata(self) -> global___MetadataV1:
        """Metadata associated with the message"""
    schema: builtins.str
    """Schema to validate the data object with - format: <message type>/<message data version>, e.g. TripCreated/1.0"""
    @property
    def data(self) -> google.protobuf.any_pb2.Any:
        """Message data"""
    def __init__(
        self,
        *,
        format_version: builtins.str = ...,
        id: builtins.str = ...,
        metadata: global___MetadataV1 | None = ...,
        schema: builtins.str = ...,
        data: google.protobuf.any_pb2.Any | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["data", b"data", "metadata", b"metadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["data", b"data", "format_version", b"format_version", "id", b"id", "metadata", b"metadata", "schema", b"schema"]) -> None: ...

global___PayloadV1 = PayloadV1



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class DeviceCreated(google.protobuf.message.Message):
    """error: no message type"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FOOBAR_FIELD_NUMBER: builtins.int
    foobar: builtins.int
    def __init__(
        self,
        *,
        foobar: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["foobar", b"foobar"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["foobar", b"foobar"]) -> None: ...

global___DeviceCreated = DeviceCreated

class TripCreatedV3(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEVICE_ID_FIELD_NUMBER: builtins.int
    device_id: builtins.str
    def __init__(
        self,
        *,
        device_id: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["device_id", b"device_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["device_id", b"device_id"]) -> None: ...

global___TripCreatedV3 = TripCreatedV3

class TripCreated(google.protobuf.message.Message):
    """error: duplicate"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FOOBAR_FIELD_NUMBER: builtins.int
    foobar: builtins.int
    def __init__(
        self,
        *,
        foobar: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["foobar", b"foobar"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["foobar", b"foobar"]) -> None: ...

global___TripCreated = TripCreated

class TripCreatedV4(google.protobuf.message.Message):
    """error: no message options"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FOOBAR_FIELD_NUMBER: builtins.int
    foobar: builtins.int
    def __init__(
        self,
        *,
        foobar: builtins.int | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["foobar", b"foobar"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["foobar", b"foobar"]) -> None: ...

global___TripCreatedV4 = TripCreatedV4



import logging
import os
from logging import Formatter
from typing import Tuple

import structlog
from hedwig.instrumentation.compat import get_hexadecimal_trace_id, get_hexadecimal_span_id, set_global_textmap
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.trace import set_tracer_provider, get_current_span
from opentelemetry.trace.propagation.tracecontext import TraceContextTextMapPropagator

from examples import base_settings, example_gcp_settings


def init():
    os.environ.setdefault("SETTINGS_MODULE", "example_gcp_settings")

    setup_logging(base_settings.LOG_LEVEL)

    set_tracer_provider(TracerProvider())

    set_global_textmap(TraceContextTextMapPropagator())


def reset_root_logger():
    root = logging.getLogger()
    for handler in root.handlers[:]:
        # Copied from `logging.shutdown`.
        try:
            handler.acquire()
            handler.flush()
            handler.close()
        except (OSError, ValueError):
            pass
        finally:
            handler.release()
        root.removeHandler(handler)


def _get_current_cloud_trace_context() -> Tuple[str, str]:
    context = get_current_span().get_span_context()
    return (
        get_hexadecimal_trace_id(context.trace_id),
        get_hexadecimal_span_id(context.span_id),
    )


class StackdriverRenderer(structlog.processors.JSONRenderer):
    """
    Render the `event_dict` using with appropriate keys renamed as suitable for Stackdriver
    See https://cloud.google.com/run/docs/logging#log-resource for more details
    """

    def __call__(self, logger, name, event_dict):
        event_dict = {
            "severity": event_dict.pop("level").upper(),
            "message": event_dict.pop("event", ""),
            **event_dict,
        }
        if "stack" in event_dict:
            event_dict["stack_trace"] = event_dict.pop("stack")
        elif "exception" in event_dict:
            event_dict["stack_trace"] = event_dict.pop("exception")
        trace_id, span_id = _get_current_cloud_trace_context()
        # Add log correlation to nest all log messages
        # beneath request log in Log Viewer.
        event_dict["logging.googleapis.com/trace"] = (
            f"projects/{example_gcp_settings.GOOGLE_CLOUD_PROJECT}/traces/{trace_id}"
        )
        event_dict["logging.googleapis.com/spanId"] = span_id
        return super().__call__(logger, name, event_dict)


def setup_logging(log_level: str = "INFO") -> None:
    # Logging setup
    # Configure struct log
    timestamper = structlog.processors.TimeStamper(fmt=Formatter.default_time_format, utc=False)
    shared_processors = [
        structlog.stdlib.add_logger_name,
        structlog.stdlib.add_log_level,
        structlog.stdlib.PositionalArgumentsFormatter(),
        timestamper,
        structlog.processors.StackInfoRenderer(),
        structlog.processors.format_exc_info,
        structlog.processors.UnicodeDecoder(),
    ]
    structlog_only_processors = [
        structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
    ]
    structlog.configure(
        processors=shared_processors + structlog_only_processors,
        context_class=dict,
        logger_factory=structlog.stdlib.LoggerFactory(),
        wrapper_class=structlog.stdlib.BoundLogger,
        cache_logger_on_first_use=True,
    )

    # Configure Python logging
    root = logging.getLogger()
    reset_root_logger()
    processor = StackdriverRenderer()
    formatter = structlog.stdlib.ProcessorFormatter(processor=processor, foreign_pre_chain=shared_processors)
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    root.addHandler(handler)
    root.setLevel(getattr(logging, log_level))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: protobuf.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'protobuf.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedwig.protobuf import options_pb2 as hedwig_dot_protobuf_dot_options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0eprotobuf.proto\x12\x05tests\x1a\x1dhedwig/protobuf/options.proto\"L\n\rTripCreatedV1\x12\x12\n\nvehicle_id\x18\x01 \x02(\t\x12\x0f\n\x07user_id\x18\x02 \x02(\t:\x16\x9a\x82\x19\x12\x08\x01\x10\x00\x1a\x0ctrip_created\"F\n\rTripCreatedV2\x12\x12\n\nvehicle_id\x18\x01 \x02(\t\x12\x0b\n\x03vin\x18\x02 \x02(\t:\x14\x9a\x82\x19\x10\x08\x02\x1a\x0ctrip_created\"M\n\x0f\x44\x65viceCreatedV1\x12\x11\n\tdevice_id\x18\x01 \x02(\t\x12\x0f\n\x07user_id\x18\x02 \x02(\t:\x16\x9a\x82\x19\x12\x08\x01\x1a\x0e\x64\x65vice.created\"?\n\x10VehicleCreatedV1\x12\x12\n\nvehicle_id\x18\x01 \x02(\t:\x17\x9a\x82\x19\x13\x08\x01\x1a\x0fvehicle_created')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protobuf_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TRIPCREATEDV1']._loaded_options = None
  _globals['_TRIPCREATEDV1']._serialized_options = b'\232\202\031\022\010\001\020\000\032\014trip_created'
  _globals['_TRIPCREATEDV2']._loaded_options = None
  _globals['_TRIPCREATEDV2']._serialized_options = b'\232\202\031\020\010\002\032\014trip_created'
  _globals['_DEVICECREATEDV1']._loaded_options = None
  _globals['_DEVICECREATEDV1']._serialized_options = b'\232\202\031\022\010\001\032\016device.created'
  _globals['_VEHICLECREATEDV1']._loaded_options = None
  _globals['_VEHICLECREATEDV1']._serialized_options = b'\232\202\031\023\010\001\032\017vehicle_created'
  _globals['_TRIPCREATEDV1']._serialized_start=56
  _globals['_TRIPCREATEDV1']._serialized_end=132
  _globals['_TRIPCREATEDV2']._serialized_start=134
  _globals['_TRIPCREATEDV2']._serialized_end=204
  _globals['_DEVICECREATEDV1']._serialized_start=206
  _globals['_DEVICECREATEDV1']._serialized_end=283
  _globals['_VEHICLECREATEDV1']._serialized_start=285
  _globals['_VEHICLECREATEDV1']._serialized_end=348
# @@protoc_insertion_point(module_scope)



import logging
import os

from hedwig.commands import requeue_dead_letter


def main():
    os.environ.setdefault("SETTINGS_MODULE", "example_settings")

    logging.basicConfig(level=logging.DEBUG)
    logging.info("Re-queuing Hedwig DLQ")
    requeue_dead_letter()


if __name__ == "__main__":
    main()



import logging
import time
import uuid
from datetime import datetime, timezone
from distutils.version import StrictVersion
from typing import Union

from opentelemetry import trace

from hedwig.models import Message

from examples import init_gcp, base_settings
from examples.models import MessageType
from examples.protos.schema_pb2 import UserCreatedV1


def main():
    init_gcp.init()

    tracer = trace.get_tracer(__name__)

    with tracer.start_as_current_span("hedwig/examples/publisher"):
        for i in range(5):
            request_id = str(uuid.uuid4())
            data = _user_data(f"U_123{i}")
            message = Message.new(
                MessageType.user_created, StrictVersion('1.0'), data, headers={'request_id': request_id}
            )
            message.publish()
            logging.info(
                f"Published message with id: '{message.id}', data: {message.data}, request id: {request_id}, "
                f"at: {datetime.now(timezone.utc)}, publish_time: {message.provider_metadata}"
            )
            time.sleep(0.2)


def _user_data(user_id) -> Union[UserCreatedV1, dict]:
    if base_settings.HEDWIG_PROTOBUF:
        data = UserCreatedV1()
        data.user_id = user_id
    else:
        data = {'user_id': user_id}
    return data


if __name__ == "__main__":
    main()



import logging
import time
from datetime import datetime, timezone

from hedwig.backends.redis import RedisMetadata
from hedwig.backends.gcp import GoogleMetadata
from hedwig.models import Message


def user_created_handler(message: Message) -> None:
    time.sleep(2)
    if message.data["user_id"].endswith("0"):
        raise ValueError
    if isinstance(message.provider_metadata, GoogleMetadata):
        publish_time = message.provider_metadata.publish_time
        delivery_attempt = message.provider_metadata.delivery_attempt
    elif isinstance(message.provider_metadata, RedisMetadata):
        publish_time = datetime.fromtimestamp(0, timezone.utc)
        delivery_attempt = message.provider_metadata.delivery_attempt
    else:
        publish_time = message.provider_metadata.sent_time
        delivery_attempt = message.provider_metadata.receive_count
    now = datetime.now(timezone.utc)
    message_creation_time = datetime.fromtimestamp(message.timestamp / 1000, timezone.utc)
    latency_s = (now - publish_time).total_seconds()
    overall_latency_s = (now - message_creation_time).total_seconds()
    logging.info(
        f'Received user created message with id: {message.id} and data: {message.data}, '
        f'publish time: {publish_time}, message time: {message.timestamp}, delivery_attempt: {delivery_attempt} '
        f'transport latency seconds: {latency_s}, overall latency seconds: {overall_latency_s}'
    )



import os

HEDWIG_CALLBACKS = {('user-created', '1.*'): 'callbacks.user_created_handler'}

HEDWIG_MESSAGE_ROUTING = {('user-created', '1.*'): 'dev-user-created-v1'}

HEDWIG_PROTOBUF = os.environ.get('HEDWIG_PROTOBUF', 'false').lower() == 'true'

if HEDWIG_PROTOBUF:
    HEDWIG_PROTOBUF_MESSAGES = ['protos.schema_pb2.UserCreatedV1', 'protos.schema_pb2.UserUpdatedV1']
    HEDWIG_DATA_VALIDATOR_CLASS = 'hedwig.validators.protobuf.ProtobufValidator'
else:
    HEDWIG_JSONSCHEMA_FILE = 'schema.json'

HEDWIG_PUBLISHER = 'myapp'

LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO").upper()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: hedwig/protobuf/options.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'hedwig/protobuf/options.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dhedwig/protobuf/options.proto\x12\x06hedwig\x1a google/protobuf/descriptor.proto\"T\n\x0eMessageOptions\x12\x15\n\rmajor_version\x18\x01 \x01(\r\x12\x15\n\rminor_version\x18\x02 \x01(\r\x12\x14\n\x0cmessage_type\x18\x03 \x01(\t:R\n\x0fmessage_options\x12\x1f.google.protobuf.MessageOptions\x18\xa3\x90\x03 \x01(\x0b\x32\x16.hedwig.MessageOptionsBI\n\x1cio.github.cloudchacho.hedwigZ)github.com/cloudchacho/hedwig-go/protobuf')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'hedwig.protobuf.options_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034io.github.cloudchacho.hedwigZ)github.com/cloudchacho/hedwig-go/protobuf'
  _globals['_MESSAGEOPTIONS']._serialized_start=75
  _globals['_MESSAGEOPTIONS']._serialized_end=159
# @@protoc_insertion_point(module_scope)
